<?php

/**
  * Addresses class, Address.php
  * Addresses management
  * @category classes
  *
  * @author PrestaShop <support@prestashop.com>
  * @copyright PrestaShop
  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
  * @version 0.8
  *
  */

class		Address extends ObjectModel
{
	/** @var integer Customer id which address belongs */
	public		$id_customer;
	
	/** @var integer Country id */
	public		$id_country;
	
	/** @var string Country name */
	public		$country;
	
	/** @var string Alias (eg. Home, Work...) */
	public		$alias;

	/** @var string Company (optional) */
	public 		$company;
	
	/** @var string Name */
	public 		$name;

	/** @var string Surname */
	public 		$surname;
	
	/** @var string Address first line */
	public 		$address1;

	/** @var string Address second line (optional) */
	public 		$address2;

	/** @var string Postal code */
	public 		$postcode;

	/** @var string City */
	public 		$city;
	
	/** @var string Any other useful information */
	public 		$other;
	
	/** @var string Phone number */
	public 		$phone;
	
	/** @var string Mobile phone number */
	public 		$phone_mobile;
	
	/** @var string Object creation date */
	public 		$date_add;

	/** @var string Object last modification date */
	public 		$date_upd;
	
	/** @var boolean True if address has been deleted (staying in database as deleted) */
	public 		$deleted = 0;

	protected	$fieldsRequired = array('id_customer', 'id_country', 'alias', 'name', 'surname', 'address1', 'postcode', 'city');
	protected	$fieldsSize = array('alias' => 32, 'company' => 32, 'name' => 32, 'surname' => 32,
									'address1' => 128, 'address2' => 128, 'postcode' => 12, 'city' => 64,
									'other' => 300, 'phone' => 16, 'phone_mobile' => 16);
	protected	$fieldsValidate = array('id_customer' => 'isUnsignedId', 'id_country' => 'isUnsignedId', 'alias' => 'isGenericName',
										'company' => 'isGenericName', 'name' => 'isName', 'surname' => 'isName',
										'address1' => 'isAddress', 'address2' => 'isAddress',
										'postcode' => 'isPostCode', 'city' => 'isCityName', 'other' => 'isGenericName',
										'phone' => 'isPhoneNumber', 'phone_mobile' => 'isPhoneNumber', 'deleted' => 'isBool');
	
	protected 	$table = 'address';
	protected 	$identifier = 'id_address';
	
	/**
	 * Build an address
	 *
	 * @param integer $id_address Existing address id in order to load object (optional)
	 */
	public	function __construct($id_address = NULL)
	{
		parent::__construct($id_address);
		
		/* Get and cache address country name */
		if ($this->id)
		{
			$result = Db::getInstance()->getRow('SELECT `name` FROM `'._DB_PREFIX_.'country_lang` WHERE `id_country` = '.intval($this->id_country));
			$this->country = $result['name'];
		}
	}
	
	public function delete()
	{
		if (!$this->isUsed())
			return parent::delete();
		else
		{
			$this->deleted = true;
			return $this->update();
		}
	}

	public function getFields()
	{
		parent::validateFields();

		$fields['id_customer'] = intval($this->id_customer);
		$fields['id_country'] = intval($this->id_country);
		$fields['alias'] = pSQL($this->alias);
		$fields['company'] = pSQL($this->company);
		$fields['name'] = pSQL(strtoupper($this->name));
		$fields['surname'] = pSQL(ucfirst(strtolower($this->surname)));
		$fields['address1'] = pSQL($this->address1);
		$fields['address2'] = pSQL($this->address2);
		$fields['postcode'] = pSQL($this->postcode);
		$fields['city'] = pSQL($this->city);
		$fields['other'] = pSQL($this->other);
		$fields['phone'] = pSQL($this->phone);
		$fields['phone_mobile'] = pSQL($this->phone_mobile);
		$fields['deleted'] = intval($this->deleted);
		$fields['date_add'] = pSQL($this->date_add);
		$fields['date_upd'] = pSQL($this->date_upd);
		return $fields;
	}

	/**
	 * Get zone id for a given address
	 *
	 * @param integer $id_address Address id for which we want to get zone id
	 * @return integer Zone id
	 */
	public static function getZoneById($id_address)
	{
		$result = Db::getInstance()->getRow('
		SELECT c.`id_zone`
		FROM `'._DB_PREFIX_.'address` a
		LEFT JOIN `'._DB_PREFIX_.'country` c ON c.`id_country` = a.`id_country`		
		WHERE a.`id_address` = '.intval($id_address));
	
		return $result['id_zone'];
	}
	
	/**
	 * Check if country is active for a given address
	 *
	 * @param integer $id_address Address id for which we want to get country status
	 * @return integer Country status
	 */
	public static function isCountryActiveById($id_address)
	{
		$result = Db::getInstance()->getRow('
		SELECT c.`active`
		FROM `'._DB_PREFIX_.'address` a
		LEFT JOIN `'._DB_PREFIX_.'country` c ON c.`id_country` = a.`id_country`		
		WHERE a.`id_address` = '.intval($id_address));
	
		return $result['active'];
	}
	
	/**
	 * Check if address is used (at least one order placed)
	 *
	 * @return integer Order count for this address
	 */
	public function isUsed()
	{
		$result = Db::getInstance()->getRow('
		SELECT COUNT(`id_order`) AS used
		FROM `'._DB_PREFIX_.'orders`
		WHERE `id_address_delivery` = '.intval($this->id).'
		OR `id_address_invoice` = '.intval($this->id));
		
		return isset($result['used']) ? $result['used'] : false;
	}
}

?>